#!/usr/bin/env python2
"""
ROS action server example
"""
from math import factorial
import numpy as np
import rospy
import actionlib  # Python package for ROS actions
"""
Next, we import the action definitions.
This is similar to messages and services. However, this time we have more "subdefinitions".
Considering our custom action called <some_action>, these additional definitions are generated:
<some_action>Action
    - base action definition
<some_action>Goal
    - this is similar to service request, i.e. the message sent to the server from caller
<some_action>Result
    - similar to service response - message sent by the server to caller upon completion
<some_action>Feedback
    - message that is sent regularly (or occasionally) by the server
    - it should inform the caller about how the action execution is going (i.e. progress)
"""
from incredible_package.msg import NumberCrunchingAction, NumberCrunchingGoal, NumberCrunchingResult, NumberCrunchingFeedback


def crunch(goal):
    """This is the main action function
    This function is called when the action is requested.

    Arguments:
        goal {type: NumberCrunchingGoal} -- the request message for the action
    """
    numbers = np.array(goal.data)
    result = np.zeros(numbers.shape)
    reps = goal.repetitions
    pause = rospy.Duration(goal.sleepTime)
    pauseCount = 0
    n = numbers.shape[0]
    total_num = result.shape[0] * reps
    success = True
    for r in range(reps):
        rospy.loginfo('Starting repetition number {}'.format(r))
        for i, num in enumerate(numbers):
            """
            The server.is_preempt_requested returns True
            if the caller request cancelation of the execution.
            You should check for this regularly and cancel
            the action (somehow) if preempt is requested.
            """
            if server.is_preempt_requested():
                server.set_preempted()  # announce that action was preempted
                success = False  # this is just a helper variable
                break
            result[i] += factorial(num)
            # send back feedback message
            # feedback contents depend on what is specified in the action definition file
            server.publish_feedback(
                NumberCrunchingFeedback(r * n + i + 1, total_num))
            rospy.sleep(pause)
            pauseCount += 1
        if not success:
            break
    if success:
        # create result/response message
        actionResult = NumberCrunchingResult(np.sum(result),
                                             pauseCount * pause.secs)
        """
        If the action succeeded, you can send response
        via the "set_succeeded" method (providing the result/response
        as the argument). There is also "set_aborted" method
        for the case where execution failed (which is different from
        being preempted by the caller).
        """
        server.set_succeeded(actionResult)
        rospy.loginfo('Action successfully completed')


if __name__ == '__main__':
    rospy.init_node('action_maker')
    # Similarly to service, advertise the action server
    server = actionlib.SimpleActionServer('number_cruncher',
                                          NumberCrunchingAction, crunch, False)
    server.start()
    rospy.spin()
