#!/usr/bin/env python2
"""
An example of a more complex ROS service provider
"""
import rospy
import numpy as np
# import service definitions
from incredible_package.srv import MatOp, MatOpResponse


def matOp(request):
    """Service request handler function
    The request message contains:
    op - transformation type ("translate", "scale" or "rotate)
    args - transformation arguments, depending on the type of trans.
    matrixIn - input transformation matrix (i.e. you can do transformation
        stacking with this service)

    Arguments:
        request {type: MatOpRequest} -- the service request message

    Returns:
        type: MatOpResponse -- the service response message
        In this case, response is a transformation matrix,
        representing the requested transformation,
        applied to the input matrix.
    """
    # The if statements below simply construct the requested trans. mat.
    if request.op == 'translate':
        mat = np.diagflat([1., 1., 1.])
        t = request.args
        mat[:2, 2] = t
    elif request.op == 'scale':
        s = list(request.args)
        mat = np.diagflat(s + [1])
    elif request.op == 'rotate':
        angle = np.deg2rad(request.args[0])
        mat = np.diagflat([0., 0., 1.])
        mat[:2, :2] = [[np.cos(angle), -np.sin(angle)],
                       [np.sin(angle), np.cos(angle)]]

    """
    Array in ROS
    In ROS, multidimensional data are usually sent as 1D arrays.
    When sending the data, the n-D arrays had to be transformed
    into 1D arrays, either by array.flatten() or array.ravel(),
    or array.reshape(1, -1) (or array.reshape(-1, 1)).
    This depends on the desired result.
    Upon receiving such flattened array, it has to be reconstructed
    into it's n-D form. The easiest way is to use np.reshape(<array>, <shape>)
    function. The <shape> is a tuple specifying number of elements
    along each dimension. If the shape is known, the array
    can be reconstructed directly. Otherwise, the shape has to be sent
    along with the array (e.g. part of the message).
    Here, we know that the transformation matrix is a 3 by 3 matrix.
    (full 2D transformation matrix) Thus, no additional info is needed.
    """
    # apply the constructed matrix to the input matrix
    matrixOut = mat.dot(np.reshape(request.matrixIn, (3, 3)))
    # construct the response message
    response = MatOpResponse(matrixOut.flatten())
    # and return it
    return response


if __name__ == "__main__":
    rospy.init_node('random_server')  # node initialization
    # advertize the service, specify name, type, and request handler
    s = rospy.Service('mat_op', MatOp, matOp)
    rospy.spin()  # hold and check for requests
