#!/usr/bin/env python2
"""
ROS TF publisher example
This node publishes a TF of the fake robot.
It also provides functionality to react to

2D navigation goals from RViz (or from any
other node that publishes the correct message
type on the appropriate topic)
"""
import rospy
import tf2_ros
import numpy as np
import tf
from geometry_msgs.msg import PoseStamped, TransformStamped, Vector3, Quaternion


class Animator():
    def __init__(self):
        # get names of the relevant frames and robot movement speed from the param server
        # "tilde" in front of the param name denotes a private parameter
        # private parameters will be named (i.e. when you
        # type rosparam list) /<node_name>/<param_name>
        # in case of namespaces, the name is preceded by namespace name
        self.speed = rospy.get_param('~speed', 0.02)
        # param name without tilde denotes global parameter
        # global parameters are named /<param_name>
        self.mapFrame = rospy.get_param('map_frame', "map")
        self.botFrame = rospy.get_param('bot_frame', "bot")
        """
        Next, we need to create listener for 2D navigation goals.
        We will use RViz capability to supply 2D Nav goals
        to provide goals for the movement of our fake bot.
        To create a 2D Nav goal in RViz, simply click the 2D Nav Goal
        button at the top of RViz or press "g" key. Then, simply click
        somewhere on the grid to issue the goal. The goal will be transmitted
        as a "stamped Pose" (PoseStamped message).
        It might be a good idea to have similar functionality implemented
        even on a real robot. Using the 2D Nav goal provides a simple
        mode of debugging.
        """
        self.goalSub = rospy.Subscriber("/move_base_simple/goal", PoseStamped, self.__onclick)

        # initial pose and goal for the robot
        self.pose = np.r_[0, 0]
        self.goal = None

        # next, we create a "TransformBroadcaster" object to broadcast the transforms
        self.broadcaster = tf2_ros.TransformBroadcaster()
        # here we just call private method to broadcast the initial pose
        self.__sendTransform()

    def __onclick(self, msg):
        """Handler for the 2D Nav goal message
        We are interested only in the position part of the nav goal,
        specifically, in the x/y coordinates.
        We assign these to the class property "goal"
        """
        self.goal = np.r_[msg.pose.position.x, msg.pose.position.y]

    def __sendTransform(self):
        """Private method for broadcasting the robot pose as TF
        """
        tf_msg = TransformStamped()  # construct empty message
        # get current ROS time and assign it to the header
        tf_msg.header.stamp = rospy.Time.now()
        # set the parent and child frames
        tf_msg.header.frame_id = self.mapFrame
        tf_msg.child_frame_id = self.botFrame

        # create 3D vector and fill it with pose value
        tf_msg.transform.translation = Vector3(*np.r_[self.pose, 0])

        # no rotation needed => default quat values
        tf_msg.transform.rotation = Quaternion(0, 0, 0, 1)

        # broadcast the transformation
        self.broadcaster.sendTransform(tf_msg)

    def update(self):
        """Main update function that moves the robot

        This method checks whether a goal is set and publishes the current
        robot position as TF. If goal exists, the robot will move towards it.
        """
        if self.goal is None:
            rospy.logwarn("No goal was set.")
        else:
            # Here, we just update the robot's position so that it gets closer to the goal
            self.pose = np.round((self.pose + np.sign(self.goal - self.pose) * self.speed) * 100) / 100
            if np.abs(self.goal - self.pose).sum() < self.speed * 1.1:
                self.pose = self.goal
                self.goal = None

        # Send the (updated) TF
        self.__sendTransform()



if __name__ == "__main__":
    rospy.init_node('animator')
    animator = Animator()
    rate = rospy.Rate(10)

    while not rospy.is_shutdown():
        animator.update()
        rate.sleep()
